# SPDX-FileCopyrightText: 2018-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

__author__ = "Nutti <nutti.metro@gmail.com>"
__status__ = "production"
__version__ = "6.6"
__date__ = "22 Apr 2022"

import bpy
from mathutils import Vector
from bpy.props import EnumProperty, BoolProperty, FloatVectorProperty
import bmesh

from .. import common
from ..utils.bl_class_registry import BlClassRegistry
from ..utils.property_class_registry import PropertyClassRegistry
from ..utils import compatibility as compat


def _is_valid_context(context):
    # 'IMAGE_EDITOR' and 'VIEW_3D' space is allowed to execute.
    # If 'View_3D' space is not allowed, you can't find option in Tool-Shelf
    # after the execution
    if not common.is_valid_space(context, ['IMAGE_EDITOR', 'VIEW_3D']):
        return False

    return True


@PropertyClassRegistry()
class _Properties:
    idname = "align_uv_cursor"

    @classmethod
    def init_props(cls, scene):
        def auvc_get_cursor_loc(self):
            area, _, space = common.get_space('IMAGE_EDITOR', 'WINDOW',
                                              'IMAGE_EDITOR')
            if compat.check_version(2, 80, 0) < 0:
                bd_size = common.get_uvimg_editor_board_size(area)
            else:
                bd_size = [1.0, 1.0]
            loc = space.cursor_location

            if bd_size[0] < 0.000001:
                cx = 0.0
            else:
                cx = loc[0] / bd_size[0]
            if bd_size[1] < 0.000001:
                cy = 0.0
            else:
                cy = loc[1] / bd_size[1]

            self['muv_align_uv_cursor_cursor_loc'] = Vector((cx, cy))
            return self.get('muv_align_uv_cursor_cursor_loc', (0.0, 0.0))

        def auvc_set_cursor_loc(self, value):
            self['muv_align_uv_cursor_cursor_loc'] = value
            area, _, space = common.get_space('IMAGE_EDITOR', 'WINDOW',
                                              'IMAGE_EDITOR')
            if compat.check_version(2, 80, 0) < 0:
                bd_size = common.get_uvimg_editor_board_size(area)
            else:
                bd_size = [1.0, 1.0]
            cx = bd_size[0] * value[0]
            cy = bd_size[1] * value[1]
            space.cursor_location = Vector((cx, cy))

        scene.muv_align_uv_cursor_enabled = BoolProperty(
            name="Align UV Cursor Enabled",
            description="Align UV Cursor is enabled",
            default=False
        )

        scene.muv_align_uv_cursor_cursor_loc = FloatVectorProperty(
            name="UV Cursor Location",
            size=2,
            precision=4,
            soft_min=-1.0,
            soft_max=1.0,
            step=1,
            default=(0.000, 0.000),
            get=auvc_get_cursor_loc,
            set=auvc_set_cursor_loc
        )
        scene.muv_align_uv_cursor_align_method = EnumProperty(
            name="Align Method",
            description="Align Method",
            default='TEXTURE',
            items=[
                ('TEXTURE', "Texture", "Align to texture"),
                ('UV', "UV", "Align to UV"),
                ('UV_SEL', "UV (Selected)", "Align to Selected UV")
            ]
        )

        scene.muv_uv_cursor_location_enabled = BoolProperty(
            name="UV Cursor Location Enabled",
            description="UV Cursor Location is enabled",
            default=False
        )

    @classmethod
    def del_props(cls, scene):
        del scene.muv_align_uv_cursor_enabled
        del scene.muv_align_uv_cursor_cursor_loc
        del scene.muv_align_uv_cursor_align_method

        del scene.muv_uv_cursor_location_enabled


@BlClassRegistry()
@compat.make_annotations
class MUV_OT_AlignUVCursor(bpy.types.Operator):

    bl_idname = "uv.muv_align_uv_cursor"
    bl_label = "Align UV Cursor"
    bl_description = "Align cursor to the center of UV island"
    bl_options = {'REGISTER', 'UNDO'}

    position = EnumProperty(
        items=(
            ('CENTER', "Center", "Align to Center"),
            ('LEFT_TOP', "Left Top", "Align to Left Top"),
            ('LEFT_MIDDLE', "Left Middle", "Align to Left Middle"),
            ('LEFT_BOTTOM', "Left Bottom", "Align to Left Bottom"),
            ('MIDDLE_TOP', "Middle Top", "Align to Middle Top"),
            ('MIDDLE_BOTTOM', "Middle Bottom", "Align to Middle Bottom"),
            ('RIGHT_TOP', "Right Top", "Align to Right Top"),
            ('RIGHT_MIDDLE', "Right Middle", "Align to Right Middle"),
            ('RIGHT_BOTTOM', "Right Bottom", "Align to Right Bottom")
        ),
        name="Position",
        description="Align position",
        default='CENTER'
    )
    base = EnumProperty(
        items=(
            ('TEXTURE', "Texture", "Align based on Texture"),
            ('UV', "UV", "Align to UV"),
            ('UV_SEL', "UV (Selected)", "Align to Selected UV")
        ),
        name="Base",
        description="Align base",
        default='TEXTURE'
    )

    @classmethod
    def poll(cls, context):
        # we can not get area/space/region from console
        if common.is_console_mode():
            return True
        return _is_valid_context(context)

    def execute(self, context):
        area, _, space = common.get_space('IMAGE_EDITOR', 'WINDOW',
                                          'IMAGE_EDITOR')
        if compat.check_version(2, 80, 0) < 0:
            bd_size = common.get_uvimg_editor_board_size(area)
        else:
            bd_size = [1.0, 1.0]

        large_value = 1e7
        if self.base == 'UV':
            objs = common.get_uv_editable_objects(context)
            no_selected_face = True
            if objs:
                max_ = Vector((-large_value, -large_value))
                min_ = Vector((large_value, large_value))
                for obj in objs:
                    bm = bmesh.from_edit_mesh(obj.data)
                    if not bm.loops.layers.uv:
                        return None
                    uv_layer = bm.loops.layers.uv.verify()

                    for f in bm.faces:
                        if (not context.tool_settings.use_uv_select_sync and
                                not f.select):
                            continue
                        for l in f.loops:
                            uv = l[uv_layer].uv
                            max_.x = max(max_.x, uv.x)
                            max_.y = max(max_.y, uv.y)
                            min_.x = min(min_.x, uv.x)
                            min_.y = min(min_.y, uv.y)
                            no_selected_face = False
            if no_selected_face:
                max_ = Vector((1.0, 1.0))
                min_ = Vector((0.0, 0.0))
            center = Vector((
                (max_.x + min_.x) / 2.0, (max_.y + min_.y) / 2.0
            ))

        # pylint: disable=R1702
        elif self.base == 'UV_SEL':
            objs = common.get_uv_editable_objects(context)
            no_selected_face = True
            if objs:
                max_ = Vector((-large_value, -large_value))
                min_ = Vector((large_value, large_value))
                for obj in objs:
                    bm = bmesh.from_edit_mesh(obj.data)
                    if not bm.loops.layers.uv:
                        return None
                    uv_layer = bm.loops.layers.uv.verify()

                    if context.tool_settings.use_uv_select_sync:
                        for v in bm.verts:
                            if not v.select:
                                continue
                            for l in v.link_loops:
                                uv = l[uv_layer].uv
                                max_.x = max(max_.x, uv.x)
                                max_.y = max(max_.y, uv.y)
                                min_.x = min(min_.x, uv.x)
                                min_.y = min(min_.y, uv.y)
                                no_selected_face = False
                    else:
                        for f in bm.faces:
                            if not f.select:
                                continue
                            for l in f.loops:
                                if not l[uv_layer].select:
                                    continue
                                uv = l[uv_layer].uv
                                max_.x = max(max_.x, uv.x)
                                max_.y = max(max_.y, uv.y)
                                min_.x = min(min_.x, uv.x)
                                min_.y = min(min_.y, uv.y)
                                no_selected_face = False
            if no_selected_face:
                max_ = Vector((1.0, 1.0))
                min_ = Vector((0.0, 0.0))
            center = Vector((
                (max_.x + min_.x) / 2.0, (max_.y + min_.y) / 2.0
            ))

        elif self.base == 'TEXTURE':
            min_ = Vector((0.0, 0.0))
            max_ = Vector((1.0, 1.0))
            center = Vector((0.5, 0.5))
        else:
            self.report({'ERROR'}, "Unknown Operation")
            return {'CANCELLED'}

        if self.position == 'CENTER':
            cx = center.x
            cy = center.y
        elif self.position == 'LEFT_TOP':
            cx = min_.x
            cy = max_.y
        elif self.position == 'LEFT_MIDDLE':
            cx = min_.x
            cy = center.y
        elif self.position == 'LEFT_BOTTOM':
            cx = min_.x
            cy = min_.y
        elif self.position == 'MIDDLE_TOP':
            cx = center.x
            cy = max_.y
        elif self.position == 'MIDDLE_BOTTOM':
            cx = center.x
            cy = min_.y
        elif self.position == 'RIGHT_TOP':
            cx = max_.x
            cy = max_.y
        elif self.position == 'RIGHT_MIDDLE':
            cx = max_.x
            cy = center.y
        elif self.position == 'RIGHT_BOTTOM':
            cx = max_.x
            cy = min_.y
        else:
            self.report({'ERROR'}, "Unknown Operation")
            return {'CANCELLED'}

        cx = cx * bd_size[0]
        cy = cy * bd_size[1]

        space.cursor_location = Vector((cx, cy))

        return {'FINISHED'}



# SPDX-FileCopyrightText: 2018-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

__author__ = "Nutti <nutti.metro@gmail.com>"
__status__ = "production"
__version__ = "6.6"
__date__ = "22 Apr 2022"

import bpy
from bpy.props import (
    FloatProperty,
    FloatVectorProperty,
    BoolProperty,
    EnumProperty,
)
from bpy.types import AddonPreferences

from . import common
from .op.clip_uv import MUV_OT_ClipUV
from .op.flip_rotate_uv import MUV_OT_FlipRotateUV
from .op.mirror_uv import MUV_OT_MirrorUV
from .op.move_uv import MUV_OT_MoveUV
from .op.unwrap_constraint import MUV_OT_UnwrapConstraint
from .op.pack_uv import MUV_OT_PackUV
from .op.smooth_uv import MUV_OT_SmoothUV
from .ui.VIEW3D_MT_uv_map import (
    MUV_MT_CopyPasteUV,
    MUV_MT_TransferUV,
    MUV_MT_WorldScaleUV,
    MUV_MT_PreserveUVAspect,
    MUV_MT_TextureLock,
    MUV_MT_TextureWrap,
    MUV_MT_TextureProjection,
    MUV_MT_UVW,
)
from .ui.VIEW3D_MT_object import MUV_MT_CopyPasteUV_Object
from .ui.IMAGE_MT_uvs import (
    MUV_MT_CopyPasteUV_UVEdit,
    MUV_MT_SelectUV,
    MUV_MT_AlignUV,
    MUV_MT_AlignUVCursor,
    MUV_MT_UVInspection,
)
from .utils.bl_class_registry import BlClassRegistry
from .utils import compatibility as compat


def view3d_uvmap_menu_fn(self, context):
    layout = self.layout
    sc = context.scene

    layout.separator()
    layout.label(text="Copy/Paste UV", icon=compat.icon('IMAGE'))
    # Copy/Paste UV
    layout.menu(MUV_MT_CopyPasteUV.bl_idname, text="Copy/Paste UV")
    # Transfer UV
    layout.menu(MUV_MT_TransferUV.bl_idname, text="Transfer UV")

    layout.separator()
    layout.label(text="UV Manipulation", icon=compat.icon('IMAGE'))
    # Flip/Rotate UV
    ops = layout.operator(MUV_OT_FlipRotateUV.bl_idname, text="Flip/Rotate UV")
    ops.seams = sc.muv_flip_rotate_uv_seams
    # Mirror UV
    ops = layout.operator(MUV_OT_MirrorUV.bl_idname, text="Mirror UV")
    ops.axis = sc.muv_mirror_uv_axis
    # Move UV
    layout.operator(MUV_OT_MoveUV.bl_idname, text="Move UV")
    # World Scale UV
    layout.menu(MUV_MT_WorldScaleUV.bl_idname, text="World Scale UV")
    # Preserve UV
    layout.menu(MUV_MT_PreserveUVAspect.bl_idname, text="Preserve UV")
    # Texture Lock
    layout.menu(MUV_MT_TextureLock.bl_idname, text="Texture Lock")
    # Texture Wrap
    layout.menu(MUV_MT_TextureWrap.bl_idname, text="Texture Wrap")
    # UV Sculpt
    layout.prop(sc, "muv_uv_sculpt_enable", text="UV Sculpt")

    layout.separator()
    layout.label(text="UV Mapping", icon=compat.icon('IMAGE'))
    # Unwrap Constraint
    ops = layout.operator(MUV_OT_UnwrapConstraint.bl_idname,
                          text="Unwrap Constraint")
    ops.u_const = sc.muv_unwrap_constraint_u_const
    ops.v_const = sc.muv_unwrap_constraint_v_const
    # Texture Projection
    layout.menu(MUV_MT_TextureProjection.bl_idname, text="Texture Projection")
    # UVW
    layout.menu(MUV_MT_UVW.bl_idname, text="UVW")


def view3d_object_menu_fn(self, _):
    layout = self.layout

    layout.separator()
    layout.label(text="Copy/Paste UV", icon=compat.icon('IMAGE'))
    # Copy/Paste UV (Among Object)
    layout.menu(MUV_MT_CopyPasteUV_Object.bl_idname, text="Copy/Paste UV")


def image_uvs_menu_fn(self, context):
    layout = self.layout
    sc = context.scene

    layout.separator()
    layout.label(text="Copy/Paste UV", icon=compat.icon('IMAGE'))
    # Copy/Paste UV (on UV/Image Editor)
    layout.menu(MUV_MT_CopyPasteUV_UVEdit.bl_idname, text="Copy/Paste UV")

    layout.separator()
    layout.label(text="UV Manipulation", icon=compat.icon('IMAGE'))
    # Clip UV
    ops = layout.operator(MUV_OT_ClipUV.bl_idname, text="Clip UV")
    ops.clip_uv_range_max = sc.muv_clip_uv_range_max
    ops.clip_uv_range_min = sc.muv_clip_uv_range_min
    # Pack UV
    ops = layout.operator(MUV_OT_PackUV.bl_idname, text="Pack UV")
    ops.allowable_center_deviation = sc.muv_pack_uv_allowable_center_deviation
    ops.allowable_size_deviation = sc.muv_pack_uv_allowable_size_deviation
    ops.accurate_island_copy = sc.muv_pack_uv_accurate_island_copy
    ops.stride = sc.muv_pack_uv_stride
    ops.apply_pack_uv = sc.muv_pack_uv_apply_pack_uv
    # Select UV
    layout.menu(MUV_MT_SelectUV.bl_idname, text="Select UV")
    # Smooth UV
    ops = layout.operator(MUV_OT_SmoothUV.bl_idname, text="Smooth")
    ops.transmission = sc.muv_smooth_uv_transmission
    ops.select = sc.muv_smooth_uv_select
    ops.mesh_infl = sc.muv_smooth_uv_mesh_infl
    # Align UV
    layout.menu(MUV_MT_AlignUV.bl_idname, text="Align UV")

    layout.separator()
    layout.label(text="Editor Enhancement", icon=compat.icon('IMAGE'))
    # Align UV Cursor
    layout.menu(MUV_MT_AlignUVCursor.bl_idname, text="Align UV Cursor")
    # UV Bounding Box
    layout.prop(sc, "muv_uv_bounding_box_show", text="UV Bounding Box")
    # UV Inspection
    layout.menu(MUV_MT_UVInspection.bl_idname, text="UV Inspection")


def add_builtin_menu():
    bpy.types.VIEW3D_MT_uv_map.append(view3d_uvmap_menu_fn)
    bpy.types.VIEW3D_MT_object.append(view3d_object_menu_fn)
    bpy.types.IMAGE_MT_uvs.append(image_uvs_menu_fn)


def remove_builtin_menu():
    bpy.types.IMAGE_MT_uvs.remove(image_uvs_menu_fn)
    bpy.types.VIEW3D_MT_object.remove(view3d_object_menu_fn)
    bpy.types.VIEW3D_MT_uv_map.remove(view3d_uvmap_menu_fn)


def set_debug_mode(self, value):
    self['enable_debug_mode'] = value


def get_debug_mode(self):
    enabled = self.get('enable_debug_mode', False)
    if enabled:
        common.enable_debugg_mode()
    else:
        common.disable_debug_mode()
    return enabled


@BlClassRegistry()
@compat.make_annotations
class MUV_Preferences(AddonPreferences):
    """Preferences class: Preferences for this add-on"""

    bl_idname = "magic_uv"

    def update_enable_builtin_menu(self, _):
        if self['enable_builtin_menu']:
            add_builtin_menu()
        else:
            remove_builtin_menu()

    # enable to add features to built-in menu
    enable_builtin_menu = BoolProperty(
        name="Built-in Menu",
        description="Enable built-in menu",
        default=True,
        update=update_enable_builtin_menu,
    )

    # enable debug mode
    enable_debug_mode = BoolProperty(
        name="Debug Mode",
        description="Enable debugging mode",
        default=False,
        set=set_debug_mode,
        get=get_debug_mode,
    )

    # for UV Sculpt
    uv_sculpt_brush_color = FloatVectorProperty(
        name="Color",
        description="Color",
        default=(1.0, 0.4, 0.4, 1.0),
        min=0.0,
        max=1.0,
        size=4,
        subtype='COLOR'
    )

    # for Overlapped UV
    uv_inspection_overlapped_color = FloatVectorProperty(
        name="Color",
        description="Color",
        default=(0.0, 0.0, 1.0, 0.3),
        min=0.0,
        max=1.0,
        size=4,
        subtype='COLOR'
    )
    uv_inspection_overlapped_color_for_v3d = FloatVectorProperty(
        name="Color (View3D)",
        description="Color in View3D",
        default=(0.0, 0.0, 1.0, 0.5),
        min=0.0,
        max=1.0,
        size=4,
        subtype='COLOR'
    )

    # for Flipped UV
    uv_inspection_flipped_color = FloatVectorProperty(
        name="Color",
        description="Color",
        default=(1.0, 0.0, 0.0, 0.3),
        min=0.0,
        max=1.0,
        size=4,
        subtype='COLOR'
    )
    uv_inspection_flipped_color_for_v3d = FloatVectorProperty(
        name="Color (View3D)",
        description="Color in View3D",
        default=(1.0, 0.0, 0.0, 0.5),
        min=0.0,
        max=1.0,
        size=4,
        subtype='COLOR'
    )

    # for Texture Projection
    texture_projection_canvas_padding = FloatVectorProperty(
        name="Canvas Padding",
        description="Canvas Padding",
        size=2,
        max=50.0,
        min=0.0,
        default=(20.0, 20.0))

    # for UV Bounding Box
    uv_bounding_box_cp_size = FloatProperty(
        name="Size",
        description="Control Point Size",
        default=6.0,
        min=3.0,
        max=100.0)
    uv_bounding_box_cp_react_size = FloatProperty(
        name="React Size",
        description="Size event fired",
        default=10.0,
        min=3.0,
        max=100.0)

    # for UI
    category = EnumProperty(
        name="Category",
        description="Preferences Category",
        items=[
            ('INFO', "Information", "Information about this add-on"),
            ('CONFIG', "Configuration", "Configuration about this add-on"),
        ],
        default='INFO'
    )
    info_desc_expanded = BoolProperty(
        name="Description",
        description="Description",
        default=False
    )
    info_loc_expanded = BoolProperty(
        name="Location",
        description="Location",
        default=False
    )
    conf_uv_sculpt_expanded = BoolProperty(
        name="UV Sculpt",
        description="UV Sculpt",
        default=False
    )
    conf_uv_inspection_expanded = BoolProperty(
        name="UV Inspection",
        description="UV Inspection",
        default=False
    )
    conf_texture_projection_expanded = BoolProperty(
        name="Texture Projection",
        description="Texture Projection",
        default=False
    )
    conf_uv_bounding_box_expanded = BoolProperty(
        name="UV Bounding Box",
        description="UV Bounding Box",
        default=False
    )

    def draw(self, _):
        layout = self.layout

        layout.row().prop(self, "category", expand=True)

        if self.category == 'INFO':
            layout.separator()

            layout.prop(
                self, "info_desc_expanded", text="Description",
                icon='DISCLOSURE_TRI_DOWN' if self.info_desc_expanded
                else 'DISCLOSURE_TRI_RIGHT')
            if self.info_desc_expanded:
                col = layout.column(align=True)
                col.label(text="Magic UV is composed of many UV editing" +
                          " features.")
                col.label(text="See tutorial page if you are new to this" +
                          " add-on.")
                col.label(text="https://github.com/nutti/Magic-UV" +
                          "/wiki/Tutorial")

            layout.prop(
                self, "info_loc_expanded", text="Location",
                icon='DISCLOSURE_TRI_DOWN' if self.info_loc_expanded
                else 'DISCLOSURE_TRI_RIGHT')
            if self.info_loc_expanded:
                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="3D View > Sidebar > " +
                         "Copy/Paste UV (Object mode)")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Copy/Paste UV (Among objects)")

                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="3D View > Sidebar > " +
                         "Copy/Paste UV (Edit mode)")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Copy/Paste UV (Among faces in 3D View)")
                col.label(text="Transfer UV")

                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="3D View > Sidebar > " +
                         "UV Manipulation (Edit mode)")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Flip/Rotate UV")
                col.label(text="Mirror UV")
                col.label(text="Move UV")
                col.label(text="World Scale UV")
                col.label(text="Preserve UV Aspect")
                col.label(text="Texture Lock")
                col.label(text="Texture Wrap")
                col.label(text="UV Sculpt")

                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="3D View > Sidebar > " +
                         "UV Manipulation (Edit mode)")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Unwrap Constraint")
                col.label(text="Texture Projection")
                col.label(text="UVW")

                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="UV/Image Editor > Sidebar > Copy/Paste UV")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Copy/Paste UV " +
                          "(Among faces in UV/Image Editor)")

                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="UV/Image Editor > Sidebar > UV Manipulation")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Align UV")
                col.label(text="Smooth UV")
                col.label(text="Select UV")
                col.label(text="Pack UV (Extension)")

                row = layout.row(align=True)
                sp = compat.layout_split(row, 0.5)
                sp.label(text="UV/Image Editor > Sidebar > " +
                         "Editor Enhancement")
                sp = compat.layout_split(sp, 1.0)
                col = sp.column(align=True)
                col.label(text="Align UV Cursor")
                col.label(text="UV Cursor Location")
                col.label(text="UV Bounding Box")
                col.label(text="UV Inspection")

        elif self.category == 'CONFIG':
            layout.separator()

            layout.prop(self, "enable_builtin_menu", text="Built-in Menu")
            layout.prop(self, "enable_debug_mode", text="Debug Mode")

            layout.separator()

            layout.prop(
                self, "conf_uv_sculpt_expanded", text="UV Sculpt",
                icon='DISCLOSURE_TRI_DOWN' if self.conf_uv_sculpt_expanded
                else 'DISCLOSURE_TRI_RIGHT')
            if self.conf_uv_sculpt_expanded:
                sp = compat.layout_split(layout, 0.05)
                col = sp.column()  # spacer
                sp = compat.layout_split(sp, 0.3)
                col = sp.column()
                col.label(text="Brush Color:")
                col.prop(self, "uv_sculpt_brush_color", text="")
                layout.separator()

            layout.prop(
                self, "conf_uv_inspection_expanded", text="UV Inspection",
                icon='DISCLOSURE_TRI_DOWN' if self.conf_uv_inspection_expanded
                else 'DISCLOSURE_TRI_RIGHT')
            if self.conf_uv_inspection_expanded:
                sp = compat.layout_split(layout, 0.05)
                col = sp.column()  # spacer
                sp = compat.layout_split(sp, 0.3)
                col = sp.column()
                col.label(text="Overlapped UV Color:")
                col.prop(self, "uv_inspection_overlapped_color", text="")
                sp = compat.layout_split(sp, 0.45)
                col = sp.column()
                col.label(text="Flipped UV Color:")
                col.prop(self, "uv_inspection_flipped_color", text="")

                sp = compat.layout_split(layout, 0.05)
                col = sp.column()  # spacer
                sp = compat.layout_split(sp, 0.3)
                col = sp.column()
                col.label(text="Overlapped UV Color (View3D):")
                col.prop(self, "uv_inspection_overlapped_color_for_v3d",
                         text="")
                sp = compat.layout_split(sp, 0.45)
                col = sp.column()
                col.label(text="Flipped UV Color (View3D):")
                col.prop(self, "uv_inspection_flipped_color_for_v3d",
                         text="")

                layout.separator()

            layout.prop(
                self, "conf_texture_projection_expanded",
                text="Texture Projection",
                icon='DISCLOSURE_TRI_DOWN'
                if self.conf_texture_projection_expanded
                else 'DISCLOSURE_TRI_RIGHT')
            if self.conf_texture_projection_expanded:
                sp = compat.layout_split(layout, 0.05)
                col = sp.column()       # spacer
                sp = compat.layout_split(sp, 0.3)
                col = sp.column()
                col.prop(self, "texture_projection_canvas_padding")
                layout.separator()

            layout.prop(
                self, "conf_uv_bounding_box_expanded", text="UV Bounding Box",
                icon='DISCLOSURE_TRI_DOWN'
                if self.conf_uv_bounding_box_expanded
                else 'DISCLOSURE_TRI_RIGHT')
            if self.conf_uv_bounding_box_expanded:
                sp = compat.layout_split(layout, 0.05)
                col = sp.column()       # spacer
                sp = compat.layout_split(sp, 0.3)
                col = sp.column()
                col.label(text="Control Point:")
                col.prop(self, "uv_bounding_box_cp_size")
                col.prop(self, "uv_bounding_box_cp_react_size")
                layout.separator()



# SPDX-FileCopyrightText: 2016-2020 by Nathan Lovato, Daniel Oakey, Razvan Radulescu, and contributors
#
# SPDX-License-Identifier: GPL-3.0-or-later

import bpy

from .utils.doc import doc_brief, doc_description, doc_idname, doc_name
from .utils.functions import convert_duration_to_frames


class POWER_SEQUENCER_OT_value_offset(bpy.types.Operator):
    """Instantly offset selected strips, either using frames or seconds. Allows to
    nudge the selection quickly, using keyboard shortcuts
    """

    doc = {
        "name": doc_name(__qualname__),
        "demo": "",
        "description": doc_description(__doc__),
        "shortcuts": [
            (
                {"type": "LEFT_ARROW", "value": "PRESS", "shift": True, "alt": True},
                {"direction": "left"},
                "Offset the selection to the left.",
            ),
            (
                {"type": "RIGHT_ARROW", "value": "PRESS", "shift": True, "alt": True},
                {"direction": "right"},
                "Offset the selection to the right.",
            ),
        ],
        "keymap": "Sequencer",
    }
    bl_idname = doc_idname(__qualname__)
    bl_label = doc["name"]
    bl_description = doc_brief(doc["description"])
    bl_options = {"REGISTER", "UNDO"}

    direction: bpy.props.EnumProperty(
        items=[
            ("left", "left", "Move the selection to the left"),
            ("right", "right", "Move the selection to the right"),
        ],
        name="Direction",
        description="Move the selection given frames or seconds",
        default="right",
        options={"HIDDEN"},
    )
    value_type: bpy.props.EnumProperty(
        items=[
            ("seconds", "Seconds", "Move with the value as seconds"),
            ("frames", "Frames", "Move with the value as frames"),
        ],
        name="Value Type",
        description="Toggle between offset in frames or seconds",
        default="seconds",
    )
    offset: bpy.props.FloatProperty(
        name="Offset",
        description="Offset amount to apply",
        default=1.0,
        step=5,
        precision=3,
    )

    @classmethod
    def poll(cls, context):
        return context.selected_sequences

    def invoke(self, context, event):
        self.offset = abs(self.offset)
        if self.direction == "left":
            self.offset *= -1.0
        return self.execute(context)

    def execute(self, context):
        offset_frames = (
            convert_duration_to_frames(context, self.offset)
            if self.value_type == "seconds"
            else self.offset
        )
        return bpy.ops.transform.seq_slide(value=(offset_frames, 0))



# SPDX-FileCopyrightText: 2020-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

import bpy
import os
from bpy.props import (
    StringProperty,
)

bl_info = {
    "name": "Import BrushSet",
    "author": "Daniel Grauer (kromar), CansecoGPC",
    "version": (1, 3, 1),
    "blender": (2, 80, 0),
    "location": "File > Import > BrushSet",
    "description": "Imports all image files from a folder.",
    "warning": '',    # used for warning icon and text in addons panel
    "doc_url": "http://wiki.blender.org/index.php/Extensions:2.5/Py/Scripts/Import-Export/BrushSet",
    "tracker_url": "https://developer.blender.org/maniphest/task/edit/form/2/",
    "category": "Import-Export",
}

fakeUser = False


def load_brush_set(dirpath):
    extensions = tuple(bpy.path.extensions_image)
    for file in os.listdir(dirpath):
        if not file.lower().endswith(extensions):
            continue

        # Load the image into data.
        path = os.path.join(dirpath, file)
        try:
            image = bpy.data.images.load(path)
        except BaseException as ex:
            print("Failed to load %r, error: %r" % (file, ex))
            continue

        image.use_fake_user = fakeUser

        # Create new texture.
        # NOTE: use the image name instead of `file` in case
        # it's encoding isn't `utf-8` compatible.
        texture = bpy.data.textures.new(image.name, 'IMAGE')
        texture.use_fake_user = fakeUser

        # Assign the image to the texture.
        texture.image = image

        print("imported:", repr(file))

    print("Brush Set imported!")

# -----------------------------------------------------------------------------


class BrushSetImporter(bpy.types.Operator):
    '''Load Brush Set'''
    bl_idname = "import_image.brushset"
    bl_label = "Import BrushSet"


    # creating a temporary filter to avoid overriding the users filters
    temp_filters: bool = True
    def draw(self, context):
        if self.temp_filters:
            context.space_data.params.use_filter = True
            context.space_data.params.use_filter_folder = True  
            context.space_data.params.use_filter_image = True  
            self.temp_filters = False

    directory: StringProperty(
        name="Directory",
        description="Directory",
        maxlen=1024,
        subtype='DIR_PATH',
    )

    def execute(self, context):
        load_brush_set(self.directory)
        return {'FINISHED'}

    def invoke(self, context, event):
        wm = context.window_manager
        wm.fileselect_add(self)
        return {'RUNNING_MODAL'}

# -----------------------------------------------------------------------------


def menu_func(self, context):
    self.layout.operator(BrushSetImporter.bl_idname, text="Brush Set")


# -----------------------------------------------------------------------------
# GUI

'''
class Brush_set_UI(bpy.types.Panel):

    bl_space_type ='USER_PREFERENCES'
    bl_label = 'Brush_Path'
    bl_region_type = 'WINDOW'
    bl_options = {'HIDE_HEADER'}

    def draw(self, context):

        scn = context.scene
        layout = self.layout
        column = layout.column(align=True)
        column.label(text='Brush Directory:')
        column.prop(scn,'filepath')
'''

# -----------------------------------------------------------------------------

classes = (
    BrushSetImporter,
)


def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    bpy.types.TOPBAR_MT_file_import.append(menu_func)


def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
    bpy.types.TOPBAR_MT_file_import.remove(menu_func)


if __name__ == "__main__":
    register()



# SPDX-FileCopyrightText: 2016-2023 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

# ----------------------------------------------------------
# support routines and general functions
# Author: Antonio Vazquez (antonioya)
#
# ----------------------------------------------------------
# noinspection PyUnresolvedReferences
import bpy
from os import path


# --------------------------------------------------------------------
# Get length Blender units
# --------------------------------------------------------------------
def get_blendunits(units):
    if bpy.context.scene.unit_settings.system == "IMPERIAL":
        return units * 0.3048
    else:
        return units


# --------------------------------------------------------------------
# Set normals
# True= faces to inside
# False= faces to outside
# --------------------------------------------------------------------
def set_normals(myobject, direction=False):
    bpy.context.view_layer.objects.active = myobject
    # go edit mode
    bpy.ops.object.mode_set(mode='EDIT')
    # select all faces
    bpy.ops.mesh.select_all(action='SELECT')
    # recalculate outside normals
    bpy.ops.mesh.normals_make_consistent(inside=direction)
    # go object mode again
    bpy.ops.object.editmode_toggle()


# --------------------------------------------------------------------
# Remove doubles
# --------------------------------------------------------------------
def remove_doubles(myobject):
    bpy.context.view_layer.objects.active = myobject
    # go edit mode
    bpy.ops.object.mode_set(mode='EDIT')
    # select all faces
    bpy.ops.mesh.select_all(action='SELECT')
    # remove
    bpy.ops.mesh.remove_doubles()
    # go object mode again
    bpy.ops.object.editmode_toggle()


# --------------------------------------------------------------------
# Set shade smooth
# --------------------------------------------------------------------
def set_smooth(myobject):
    # deactivate others
    for o in bpy.data.objects:
        if o.select_get() is True:
            o.select_set(False)

    myobject.select_set(True)
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.shade_smooth()


# --------------------------------------------------------------------
# Add modifier (subdivision)
# --------------------------------------------------------------------
def set_modifier_subsurf(myobject):
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.modifier_add(type='SUBSURF')
        for mod in myobject.modifiers:
            if mod.type == 'SUBSURF':
                mod.levels = 2


# --------------------------------------------------------------------
# Add modifier (mirror)
# --------------------------------------------------------------------
def set_modifier_mirror(myobject, axis="Y"):
    bpy.ops.object.select_all(action='DESELECT')
    myobject.select_set(True)
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.modifier_add(type='MIRROR')
        for mod in myobject.modifiers:
            if mod.type == 'MIRROR':
                if axis == "X":
                    mod.use_axis[0] = True
                else:
                    mod.use__axis[0] = False

                if axis == "Y":
                    mod.use_axis[1] = True
                else:
                    mod.use_axis[1] = False

                if axis == "Z":
                    mod.use_axis[2] = True
                else:
                    mod.use_axis[2] = False

                mod.use_clip = True


# --------------------------------------------------------------------
# Add modifier (array)
# --------------------------------------------------------------------
def set_modifier_array(myobject, axis, move, repeat, fix=False, fixmove=0, zmove=0):
    bpy.ops.object.select_all(action='DESELECT')
    myobject.select_set(True)
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.modifier_add(type='ARRAY')
        for mod in myobject.modifiers:
            if mod.type == 'ARRAY':
                if mod.name == "Array":
                    mod.name = "Array_" + axis
                    mod.count = repeat
                    mod.use_constant_offset = fix
                    if axis == "X":
                        mod.relative_offset_displace[0] = move
                        mod.constant_offset_displace[0] = fixmove
                        mod.relative_offset_displace[1] = 0.0
                        mod.constant_offset_displace[1] = 0.0
                        mod.relative_offset_displace[2] = 0.0
                        mod.constant_offset_displace[2] = zmove

                    if axis == "Y":
                        mod.relative_offset_displace[0] = 0.0
                        mod.constant_offset_displace[0] = 0.0
                        mod.relative_offset_displace[1] = move
                        mod.constant_offset_displace[1] = fixmove
                        mod.relative_offset_displace[2] = 0.0
                        mod.constant_offset_displace[2] = 0.0


# --------------------------------------------------------------------
# Add modifier (curve)
# --------------------------------------------------------------------
def set_modifier_curve(myobject, mycurve):
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.modifier_add(type='CURVE')
        for mod in myobject.modifiers:
            if mod.type == 'CURVE':
                mod.deform_axis = 'POS_X'
                mod.object = mycurve


# --------------------------------------------------------------------
# Add modifier (solidify)
# --------------------------------------------------------------------
def set_modifier_solidify(myobject, width):
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.modifier_add(type='SOLIDIFY')
        for mod in myobject.modifiers:
            if mod.type == 'SOLIDIFY':
                mod.thickness = width
                mod.use_even_offset = True
                mod.use_quality_normals = True
                break


# --------------------------------------------------------------------
# Add modifier (boolean)
# --------------------------------------------------------------------
def set_modifier_boolean(myobject, bolobject):
    boolean_modifier = myobject.modifiers.new("", 'BOOLEAN')
    boolean_modifier.operation = 'DIFFERENCE'
    boolean_modifier.object = bolobject


# --------------------------------------------------------------------
# Set material to object
# --------------------------------------------------------------------
def set_material(myobject, mymaterial):
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        myobject.data.materials.append(mymaterial)


# --------------------------------------------------------------------
# Set material to selected faces
# --------------------------------------------------------------------
def set_material_faces(myobject, idx):
    bpy.context.view_layer.objects.active = myobject
    myobject.select_set(True)
    bpy.context.object.active_material_index = idx
    if bpy.context.view_layer.objects.active.name == myobject.name:
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.object.material_slot_assign()
        # Deselect
        bpy.ops.mesh.select_all(action='DESELECT')
        bpy.ops.object.mode_set(mode='OBJECT')


# --------------------------------------------------------------------
# Select faces
# --------------------------------------------------------------------
def select_faces(myobject, selface, clear):
    myobject.select_set(True)
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        # deselect everything
        if clear:
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='DESELECT')

        # reselect the originally selected face
        bpy.ops.object.mode_set(mode='OBJECT')
        myobject.data.polygons[selface].select = True


# --------------------------------------------------------------------
# Select vertices
# --------------------------------------------------------------------
def select_vertices(myobject, selvertices, clear=True):
    myobject.select_set(True)
    bpy.context.view_layer.objects.active = myobject
    if bpy.context.view_layer.objects.active.name == myobject.name:
        # deselect everything
        if clear:
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='DESELECT')

        # Select Vertices
        bpy.ops.object.mode_set(mode='EDIT', toggle=False)
        sel_mode = bpy.context.tool_settings.mesh_select_mode

        bpy.context.tool_settings.mesh_select_mode = [True, False, False]
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

        for i in selvertices:
            myobject.data.vertices[i].select = True

        bpy.ops.object.mode_set(mode='EDIT', toggle=False)
        bpy.context.tool_settings.mesh_select_mode = sel_mode
        bpy.ops.object.mode_set(mode='OBJECT')


# --------------------------------------------------------------------
# Mark Seam
# --------------------------------------------------------------------
def mark_seam(myobject):
    # noinspection PyBroadException
    try:
        myobject.select_set(True)
        bpy.context.view_layer.objects.active = myobject
        if bpy.context.view_layer.objects.active.name == myobject.name:
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            bpy.ops.mesh.mark_seam()
            bpy.ops.object.mode_set(mode='OBJECT')
    except:
        bpy.ops.object.mode_set(mode='OBJECT')


# --------------------------------------------------------------------
# Unwrap mesh
# --------------------------------------------------------------------
def unwrap_mesh(myobject, allfaces=True):
    # noinspection PyBroadException
    try:
        myobject.select_set(True)
        bpy.context.view_layer.objects.active = myobject
        if bpy.context.view_layer.objects.active.name == myobject.name:
            # Unwrap
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            if allfaces is True:
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.mesh.select_all()
            bpy.ops.uv.unwrap()
            bpy.ops.object.mode_set(mode='OBJECT')
    except:
        bpy.ops.object.mode_set(mode='OBJECT')


# --------------------------------------------------------------------
# Get Node Index(multilanguage support)
# --------------------------------------------------------------------
def get_node_index(nodes, datatype):
    idx = 0
    for m in nodes:
        if m.type == datatype:
            return idx
        idx += 1

    # by default
    return 1


# --------------------------------------------------------------------
# Create cycles diffuse material
# --------------------------------------------------------------------
def create_diffuse_material(matname, replace, r, g, b, rv=0.8, gv=0.8, bv=0.8, mix=0.1, twosides=False):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.diffuse_color = (rv, gv, bv, 1.0)  # viewport color
    mat.use_nodes = True
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes.new('ShaderNodeBsdfDiffuse')
    node.name = 'Diffuse BSDF'
    node.inputs[0].default_value = [r, g, b, 1]
    node.location = 200, 320

    node = nodes.new('ShaderNodeBsdfGlossy')
    node.name = 'Glossy_0'
    node.location = 200, 0

    node = nodes.new('ShaderNodeMixShader')
    node.name = 'Mix_0'
    node.inputs[0].default_value = mix
    node.location = 500, 160

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 1100, 160

    # Connect nodes
    outn = nodes['Diffuse BSDF'].outputs[0]
    inn = nodes['Mix_0'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Glossy_0'].outputs[0]
    inn = nodes['Mix_0'].inputs[2]
    mat.node_tree.links.new(outn, inn)

    if twosides is False:
        outn = nodes['Mix_0'].outputs[0]
        inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
        mat.node_tree.links.new(outn, inn)

    if twosides is True:
        node = nodes.new('ShaderNodeNewGeometry')
        node.name = 'Input_1'
        node.location = -80, -70

        node = nodes.new('ShaderNodeBsdfDiffuse')
        node.name = 'Diffuse_1'
        node.inputs[0].default_value = [0.30, 0.30, 0.30, 1]
        node.location = 200, -280

        node = nodes.new('ShaderNodeMixShader')
        node.name = 'Mix_1'
        node.inputs[0].default_value = mix
        node.location = 800, -70

        outn = nodes['Input_1'].outputs[6]
        inn = nodes['Mix_1'].inputs[0]
        mat.node_tree.links.new(outn, inn)

        outn = nodes['Diffuse_1'].outputs[0]
        inn = nodes['Mix_1'].inputs[2]
        mat.node_tree.links.new(outn, inn)

        outn = nodes['Mix_0'].outputs[0]
        inn = nodes['Mix_1'].inputs[1]
        mat.node_tree.links.new(outn, inn)

        outn = nodes['Mix_1'].outputs[0]
        inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
        mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles translucent material
# --------------------------------------------------------------------
def create_translucent_material(matname, replace, r, g, b, rv=0.8, gv=0.8, bv=0.8, mix=0.1):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.diffuse_color = (rv, gv, bv, 1.0)  # viewport color
    mat.use_nodes = True
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes.new('ShaderNodeBsdfDiffuse')
    node.name = 'Diffuse BSDF'
    node.inputs[0].default_value = [r, g, b, 1]
    node.location = 200, 320

    node = nodes.new('ShaderNodeBsdfTranslucent')
    node.name = 'Translucent_0'
    node.location = 200, 0

    node = nodes.new('ShaderNodeMixShader')
    node.name = 'Mix_0'
    node.inputs[0].default_value = mix
    node.location = 500, 160

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 1100, 160

    # Connect nodes
    outn = nodes['Diffuse BSDF'].outputs[0]
    inn = nodes['Mix_0'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Translucent_0'].outputs[0]
    inn = nodes['Mix_0'].inputs[2]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Mix_0'].outputs[0]
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles glass material
# --------------------------------------------------------------------
def create_glass_material(matname, replace, rv=0.333, gv=0.342, bv=0.9):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    mat.diffuse_color = (rv, gv, bv, 1.0)
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes[get_node_index(nodes, 'BSDF_DIFFUSE')]
    mat.node_tree.nodes.remove(node)  # remove not used

    node = nodes.new('ShaderNodeLightPath')
    node.name = 'Light_0'
    node.location = 10, 160

    node = nodes.new('ShaderNodeBsdfRefraction')
    node.name = 'Refraction_0'
    node.inputs[2].default_value = 1  # IOR 1.0
    node.location = 250, 400

    node = nodes.new('ShaderNodeBsdfGlossy')
    node.name = 'Glossy_0'
    node.location = 250, 100

    node = nodes.new('ShaderNodeBsdfTransparent')
    node.name = 'Transparent_0'
    node.location = 500, 10

    node = nodes.new('ShaderNodeMixShader')
    node.name = 'Mix_0'
    node.inputs[0].default_value = 0.035
    node.location = 500, 160

    node = nodes.new('ShaderNodeMixShader')
    node.name = 'Mix_1'
    node.inputs[0].default_value = 0.1
    node.location = 690, 290

    node = nodes.new('ShaderNodeOutputMaterial')
    node.name = 'OUTPUT_MATERIAL'
    node.location = 920, 290

    # Connect nodes
    outn = nodes['Light_0'].outputs[1]
    inn = nodes['Mix_1'].inputs[0]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Refraction_0'].outputs[0]
    inn = nodes['Mix_0'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Glossy_0'].outputs[0]
    inn = nodes['Mix_0'].inputs[2]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Mix_0'].outputs[0]
    inn = nodes['Mix_1'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Transparent_0'].outputs[0]
    inn = nodes['Mix_1'].inputs[2]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Mix_1'].outputs[0]
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# ---------------------------------------------
# Create cycles transparents material
# --------------------------------------------------------------------
def create_transparent_material(matname, replace, r=1, g=1, b=1, alpha=0):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    mat.diffuse_color = (r, g, b, 1.0)
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes[get_node_index(nodes, 'BSDF_DIFFUSE')]
    mat.node_tree.nodes.remove(node)  # remove not used

    node = nodes.new('ShaderNodeBsdfTransparent')
    node.name = 'Transparent_0'
    node.location = 250, 160
    node.inputs[0].default_value = [r, g, b, alpha]

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 700, 160

    # Connect nodes
    outn = nodes['Transparent_0'].outputs[0]
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles glossy material
# --------------------------------------------------------------------
def create_glossy_material(matname, replace, r, g, b, rv=0.578, gv=0.555, bv=0.736, rvalue=0.2):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    mat.diffuse_color = (rv, gv, bv, 1.0)
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes[get_node_index(nodes, 'BSDF_DIFFUSE')]
    mat.node_tree.nodes.remove(node)  # remove not used

    node = nodes.new('ShaderNodeBsdfGlossy')
    node.name = 'Glossy_0'
    node.inputs[0].default_value = [r, g, b, 1]
    node.inputs[1].default_value = rvalue
    node.location = 200, 160

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 700, 160

    # Connect nodes
    outn = nodes['Glossy_0'].outputs[0]
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles emission material
# --------------------------------------------------------------------
def create_emission_material(matname, replace, r, g, b, energy):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes[get_node_index(nodes, 'BSDF_DIFFUSE')]
    mat.node_tree.nodes.remove(node)  # remove not used

    node = nodes.new('ShaderNodeEmission')
    node.name = 'Emission_0'
    node.inputs[0].default_value = [r, g, b, 1]
    node.inputs[1].default_value = energy
    node.location = 200, 160

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 700, 160

    # Connect nodes
    outn = nodes['Emission_0'].outputs[0]
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles glass material
# --------------------------------------------------------------------
def create_old_glass_material(matname, replace, rv=0.352716, gv=0.760852, bv=0.9):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    mat.diffuse_color = (rv, gv, bv, 1.0)
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes[get_node_index(nodes, 'BSDF_DIFFUSE')]
    mat.node_tree.nodes.remove(node)  # remove not used

    node = nodes.new('ShaderNodeLightPath')
    node.name = 'Light_0'
    node.location = 10, 160

    node = nodes.new('ShaderNodeBsdfGlass')
    node.name = 'Glass_0'
    node.location = 250, 300

    node = nodes.new('ShaderNodeBsdfTransparent')
    node.name = 'Transparent_0'
    node.location = 250, 0

    node = nodes.new('ShaderNodeMixShader')
    node.name = 'Mix_0'
    node.inputs[0].default_value = 0.1
    node.location = 500, 160

    node = nodes.new('ShaderNodeMixShader')
    node.name = 'Mix_1'
    node.inputs[0].default_value = 0.1
    node.location = 690, 290

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 920, 290

    # Connect nodes
    outn = nodes['Light_0'].outputs[1]
    inn = nodes['Mix_0'].inputs[0]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Light_0'].outputs[2]
    inn = nodes['Mix_1'].inputs[0]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Glass_0'].outputs[0]
    inn = nodes['Mix_0'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Transparent_0'].outputs[0]
    inn = nodes['Mix_0'].inputs[2]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Mix_0'].outputs[0]
    inn = nodes['Mix_1'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Mix_1'].outputs[0]
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles brick texture material
# --------------------------------------------------------------------
def create_brick_material(matname, replace, r, g, b, rv=0.8, gv=0.636, bv=0.315):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    mat.diffuse_color = (rv, gv, bv, 1.0)
    nodes = mat.node_tree.nodes

    # support for multilanguage
    principled_node = nodes[get_node_index(nodes, 'BSDF_PRINCIPLED')]

    principled_node.inputs[0].default_value = [r, g, b, 1]
    principled_node.location = 500, 160

    output_node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    output_node.location = 700, 160

    brick_node = nodes.new('ShaderNodeTexBrick')
    brick_node.name = 'Brick_0'
    brick_node.inputs[3].default_value = [0.407, 0.411, 0.394, 1]  # mortar color
    brick_node.inputs[4].default_value = 3  # scale
    brick_node.inputs[5].default_value = 0.001  # mortar
    brick_node.inputs[7].default_value = 0.60  # size_w
    brick_node.inputs[8].default_value = 0.30  # size_h
    brick_node.location = 300, 160

    rgb_node = nodes.new('ShaderNodeRGB')
    rgb_node.name = 'RGB_0'
    rgb_node.outputs[0].default_value = [r, g, b, 1]
    rgb_node.location = 70, 160

    # Connect nodes
    outn = rgb_node.outputs['Color']
    inn = brick_node.inputs['Color1']
    mat.node_tree.links.new(outn, inn)

    inn = brick_node.inputs['Color2']
    mat.node_tree.links.new(outn, inn)

    outn = brick_node.outputs['Color']
    inn = principled_node.inputs['Base Color']
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Create cycles fabric texture material
# --------------------------------------------------------------------
def create_fabric_material(matname, replace, r, g, b, rv=0.8, gv=0.636, bv=0.315):
    # Avoid duplicate materials
    if replace is False:
        matlist = bpy.data.materials
        for m in matlist:
            if m.name == matname:
                return m
    # Create material
    mat = bpy.data.materials.new(matname)
    mat.use_nodes = True
    mat.diffuse_color = (rv, gv, bv, 1.0)
    nodes = mat.node_tree.nodes

    # support for multilanguage
    node = nodes.new('ShaderNodeBsdfDiffuse')
    node.name = 'Diffuse BSDF'
    node.inputs[0].default_value = [r, g, b, 1]
    node.location = 810, 270

    node = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')]
    node.location = 1210, 320

    node = nodes.new('ShaderNodeTexCoord')
    node.name = 'UVCoordinates'
    node.location = 26, 395

    node = nodes.new('ShaderNodeMapping')
    node.name = 'UVMapping'
    node.location = 266, 380
    node.inputs['Scale'].default_value[0] = 1000
    node.inputs['Scale'].default_value[1] = 1000
    node.inputs['Scale'].default_value[2] = 1000

    # ===========================================================================
    # Image texture
    # ===========================================================================
    # Load image file.

    realpath = path.join(path.dirname(__file__), "images", "fabric_diffuse.png")
    print("Loading: " + realpath)
    try:
        img = bpy.data.images.load(realpath)
    except:
        raise NameError("Cannot load image %s" % realpath)

    # Create image texture from image
    ctex = bpy.data.textures.new('ColorTex', type='IMAGE')
    ctex.image = img

    node = nodes.new('ShaderNodeTexImage')
    node.name = 'Image1'
    node.image = ctex.image
    node.location = 615, 350

    node = nodes.new('ShaderNodeBsdfTransparent')
    node.name = 'Transparent1'
    node.location = 810, 395
    node.inputs[0].default_value = [r, g, b, 1]

    node = nodes.new('ShaderNodeAddShader')
    node.name = 'Add1'
    node.location = 1040, 356

    # Connect nodes
    outn = nodes['UVCoordinates'].outputs['UV']
    inn = nodes['UVMapping'].inputs['Vector']
    mat.node_tree.links.new(outn, inn)

    outn = nodes['UVMapping'].outputs['Vector']
    inn = nodes['Image1'].inputs['Vector']
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Image1'].outputs['Color']
    inn = nodes['Diffuse BSDF'].inputs['Color']
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Transparent1'].outputs['BSDF']
    inn = nodes['Add1'].inputs[0]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Diffuse BSDF'].outputs['BSDF']
    inn = nodes['Add1'].inputs[1]
    mat.node_tree.links.new(outn, inn)

    outn = nodes['Add1'].outputs['Shader']
    inn = nodes[get_node_index(nodes, 'OUTPUT_MATERIAL')].inputs[0]
    mat.node_tree.links.new(outn, inn)

    return mat


# --------------------------------------------------------------------
# Copy bin file
# --------------------------------------------------------------------
def copy_binfile(fromfile, tofile):
    with open(fromfile, 'rb') as f1:
        with open(tofile, 'wb') as f2:
            while True:
                mybytes = f1.read(1024)
                if mybytes:
                    f2.write(mybytes)
                else:
                    break


# --------------------------------------------------------------------
# Parent object (keep positions)
# --------------------------------------------------------------------
def parentobject(parentobj, childobj):
    # noinspection PyBroadException
    try:
        bpy.ops.object.select_all(action='DESELECT')
        bpy.context.view_layer.objects.active = parentobj
        parentobj.select_set(True)
        childobj.select_set(True)
        bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)
        return True
    except:
        return False


# ------------------------------------------------------------------------------
# Create control box
#
# objName: Object name
# x: size x axis
# y: size y axis
# z: size z axis
# tube: True create a tube, False only sides
# ------------------------------------------------------------------------------
def create_control_box(objname, x, y, z, tube=True):
    myvertex = [(-x / 2, 0, 0.0),
                (-x / 2, y, 0.0),
                (x / 2, y, 0.0),
                (x / 2, 0, 0.0),
                (-x / 2, 0, z),
                (-x / 2, y, z),
                (x / 2, y, z),
                (x / 2, 0, z)]

    if tube is True:
        myfaces = [(0, 1, 2, 3), (0, 4, 5, 1), (1, 5, 6, 2), (3, 7, 4, 0), (2, 6, 7, 3), (5, 4, 7, 6)]
    else:
        myfaces = [(0, 4, 5, 1), (2, 6, 7, 3)]

    mesh = bpy.data.meshes.new(objname)
    myobject = bpy.data.objects.new(objname, mesh)

    myobject.location = bpy.context.scene.cursor.location
    bpy.context.collection.objects.link(myobject)

    mesh.from_pydata(myvertex, [], myfaces)
    mesh.update(calc_edges=True)

    return myobject


# ------------------------------------------------------------------------------
# Remove all children objects
# ------------------------------------------------------------------------------
def remove_children(myobject):
    # Remove children
    for child in myobject.children:
        # noinspection PyBroadException
        try:
            # noinspection PyBroadException
            try:
                # remove child relationship
                for grandchild in child.children:
                    grandchild.parent = None
                # remove modifiers
                for mod in child.modifiers:
                    bpy.ops.object.modifier_remove(name=mod.name)
            except:
                pass
            # clear child data
            if child.type == 'MESH':
                old = child.data
                child.select_set(True)
                bpy.ops.object.delete()
                bpy.data.meshes.remove(old)
            if child.type == 'CURVE':
                child.select_set(True)
                bpy.ops.object.delete()
        except:
            pass


# --------------------------------------------------------------------
# Get all parents
# --------------------------------------------------------------------
def get_allparents(myobj):
    obj = myobj
    mylist = []
    while obj.parent is not None:
        mylist.append(obj)
        objp = obj.parent
        obj = objp

    mylist.append(obj)

    return mylist


# --------------------------------------------------------------------
# Verify all faces are in vertice group to avoid Blander crash
#
# Review the faces array and remove any vertex out of the range
# this avoid any bug that can appear avoiding Blender crash
# --------------------------------------------------------------------
def check_mesh_errors(myvertices, myfaces):
    vmax = len(myvertices)

    f = 0
    for face in myfaces:
        for v in face:
            if v < 0 or v > vmax:
                print("Face=" + str(f) + "->removed vertex=" + str(v))
                myfaces[f].remove(v)
        f += 1

    return myfaces



# SPDX-FileCopyrightText: 2016-2020 by Nathan Lovato, Daniel Oakey, Razvan Radulescu, and contributors
#
# SPDX-License-Identifier: GPL-3.0-or-later

import bpy

from .utils.doc import doc_name, doc_idname, doc_brief, doc_description


class POWER_SEQUENCER_OT_playback_speed_set(bpy.types.Operator):
    """
    Change the playback_speed property using an operator property. Used with keymaps
    """

    doc = {
        "name": doc_name(__qualname__),
        "demo": "",
        "description": doc_description(__doc__),
        "shortcuts": [
            ({"type": "ONE", "ctrl": True, "value": "PRESS"}, {"speed": "NORMAL"}, "Speed to 1x"),
            ({"type": "TWO", "ctrl": True, "value": "PRESS"}, {"speed": "DOUBLE"}, "Speed to 2x"),
            ({"type": "THREE", "ctrl": True, "value": "PRESS"}, {"speed": "TRIPLE"}, "Speed to 3x"),
        ],
        "keymap": "Sequencer",
    }
    bl_idname = doc_idname(__qualname__)
    bl_label = doc["name"]
    bl_description = doc_brief(doc["description"])
    bl_options = {"REGISTER"}

    speed: bpy.props.EnumProperty(
        items=[
            ("NORMAL", "Normal (1x)", ""),
            ("DOUBLE", "Double (2x)", ""),
            ("TRIPLE", "Triple (3x)", ""),
        ],
        name="Speed",
        description="Change the playback speed",
        default="DOUBLE",
    )

    @classmethod
    def poll(cls, context):
        return context.sequences

    def execute(self, context):
        context.scene.power_sequencer.playback_speed = self.speed
        return {"FINISHED"}



# SPDX-FileCopyrightText: 2016-2020 by Nathan Lovato, Daniel Oakey, Razvan Radulescu, and contributors
#
# SPDX-License-Identifier: GPL-3.0-or-later

import bpy

from .utils.global_settings import SequenceTypes
from .utils.doc import doc_name, doc_idname, doc_brief, doc_description


class POWER_SEQUENCER_OT_grab_sequence_handles(bpy.types.Operator):
    """
    *brief* Grabs the sequence's handle based on the mouse position


    Extends the sequence based on the mouse position. If the cursor is to the
    right of the sequence's middle, it moves the right handle. If it's on the
    left side, it moves the left handle.
    """

    doc = {
        "name": doc_name(__qualname__),
        "demo": "",
        "description": doc_description(__doc__),
        "shortcuts": [
            ({"type": "G", "value": "PRESS", "shift": True}, {}, "Grab sequence handles")
        ],
        "keymap": "Sequencer",
    }
    bl_idname = doc_idname(__qualname__)
    bl_label = doc["name"]
    bl_description = doc_brief(doc["description"])
    bl_options = {"REGISTER", "UNDO"}

    always_find_closest: bpy.props.BoolProperty(name="Always find closest", default=False)
    frame: bpy.props.FloatProperty(name="Frame", default=-1.0, options={"HIDDEN"})
    channel: bpy.props.FloatProperty(name="Channel", default=-1.0, options={"HIDDEN"})

    @classmethod
    def poll(cls, context):
        return context.sequences

    def invoke(self, context, event):
        self.frame, self.channel = context.region.view2d.region_to_view(
            x=event.mouse_region_x, y=event.mouse_region_y
        )
        return self.execute(context)

    def execute(self, context):
        selection = context.selected_sequences
        if self.always_find_closest or not selection:
            if self.frame == -1:
                return {"CANCELLED"}
            bpy.ops.power_sequencer.select_closest_to_mouse(frame=self.frame, channel=self.channel)
            for s in context.selected_sequences:
                self.select_closest_handle(s)
        else:
            bpy.ops.sequencer.select_all(action="DESELECT")
            for s in selection:
                if s.type in SequenceTypes.EFFECT and not s.type == "COLOR":
                    self.select_closest_handle(s.input_1)
                    try:
                        self.select_closest_handle(s.input_2)
                    except AttributeError:
                        pass
                else:
                    self.select_closest_handle(s)
        return bpy.ops.transform.seq_slide("INVOKE_DEFAULT")

    def select_closest_handle(self, sequence):
        middle = sequence.frame_final_start + sequence.frame_final_duration / 2
        if self.frame >= middle:
            sequence.select_right_handle = True
        else:
            sequence.select_left_handle = True
        sequence.select = True



# SPDX-FileCopyrightText: 2019-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

# This try block allows for the script to psudo-intelligently select the
# appropriate random to use.  If Numpy's random is present it will use that.
# If Numpy's random is not present, it will through a "module not found"
# exception and instead use the slower built-in random that Python has.
try:
    from numpy.random import random_integers as randint
    from numpy.random import normal as gauss
    from numpy.random import (
        beta,
        uniform,
    )
except:
    from random import (
        randint,
        gauss,
        uniform,
    )
    from random import betavariate as beta

from .utils import skewedGauss


def randomizeTexture(texture, level=1):
    '''
    Set the values for a texture from parameters.

    param: texture - bpy.data.texture to modify.
    level   - designated tweaked settings to use
    -> Below 10 is a displacement texture
    -> Between 10 and 20 is a base material texture
    '''
    noises = ['BLENDER_ORIGINAL', 'ORIGINAL_PERLIN', 'IMPROVED_PERLIN',
              'VORONOI_F1', 'VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4',
              'VORONOI_F2_F1', 'VORONOI_CRACKLE']
    if texture.type == 'CLOUDS':
        if randint(0, 1) == 0:
            texture.noise_type = 'SOFT_NOISE'
        else:
            texture.noise_type = 'HARD_NOISE'
        if level != 11:
            tempInt = randint(0, 6)
        else:
            tempInt = randint(0, 8)
        texture.noise_basis = noises[tempInt]
        texture.noise_depth = 8

        if level == 0:
            texture.noise_scale = gauss(0.625, 1 / 24)
        elif level == 2:
            texture.noise_scale = 0.15
        elif level == 11:
            texture.noise_scale = gauss(0.5, 1 / 24)

            if texture.noise_basis in ['BLENDER_ORIGINAL', 'ORIGINAL_PERLIN',
                                       'IMPROVED_PERLIN', 'VORONOI_F1']:
                texture.intensity = gauss(1, 1 / 6)
                texture.contrast = gauss(4, 1 / 3)
            elif texture.noise_basis in ['VORONOI_F2', 'VORONOI_F3', 'VORONOI_F4']:
                texture.intensity = gauss(0.25, 1 / 12)
                texture.contrast = gauss(2, 1 / 6)
            elif texture.noise_basis == 'VORONOI_F2_F1':
                texture.intensity = gauss(0.5, 1 / 6)
                texture.contrast = gauss(2, 1 / 6)
            elif texture.noise_basis == 'VORONOI_CRACKLE':
                texture.intensity = gauss(0.5, 1 / 6)
                texture.contrast = gauss(2, 1 / 6)
    elif texture.type == 'MUSGRAVE':
        # musgraveType = ['MULTIFRACTAL', 'RIDGED_MULTIFRACTAL',
        #                 'HYBRID_MULTIFRACTAL', 'FBM', 'HETERO_TERRAIN']
        texture.musgrave_type = 'MULTIFRACTAL'
        texture.dimension_max = abs(gauss(0, 0.6)) + 0.2
        texture.lacunarity = beta(3, 8) * 8.2 + 1.8

        if level == 0:
            texture.noise_scale = gauss(0.625, 1 / 24)
            texture.noise_intensity = 0.2
            texture.octaves = 1.0
        elif level == 2:
            texture.intensity = gauss(1, 1 / 6)
            texture.contrast = 0.2
            texture.noise_scale = 0.15
            texture.octaves = 8.0
        elif level == 10:
            texture.intensity = gauss(0.25, 1 / 12)
            texture.contrast = gauss(1.5, 1 / 6)
            texture.noise_scale = 0.5
            texture.octaves = 8.0
        elif level == 12:
            texture.octaves = uniform(1, 3)
        elif level > 12:
            texture.octaves = uniform(2, 8)
        else:
            texture.intensity = gauss(1, 1 / 6)
            texture.contrast = 0.2
            texture.octaves = 8.0
    elif texture.type == 'DISTORTED_NOISE':
        tempInt = randint(0, 8)
        texture.noise_distortion = noises[tempInt]
        tempInt = randint(0, 8)
        texture.noise_basis = noises[tempInt]
        texture.distortion = skewedGauss(2.0, 2.6666, (0.0, 10.0), False)

        if level == 0:
            texture.noise_scale = gauss(0.625, 1 / 24)
        elif level == 2:
            texture.noise_scale = 0.15
        elif level >= 12:
            texture.noise_scale = gauss(0.2, 1 / 48)
    elif texture.type == 'STUCCI':
        stucciTypes = ['PLASTIC', 'WALL_IN', 'WALL_OUT']
        if randint(0, 1) == 0:
            texture.noise_type = 'SOFT_NOISE'
        else:
            texture.noise_type = 'HARD_NOISE'
        tempInt = randint(0, 2)
        texture.stucci_type = stucciTypes[tempInt]

        if level == 0:
            tempInt = randint(0, 6)
            texture.noise_basis = noises[tempInt]
            texture.noise_scale = gauss(0.625, 1 / 24)
        elif level == 2:
            tempInt = randint(0, 6)
            texture.noise_basis = noises[tempInt]
            texture.noise_scale = 0.15
        elif level >= 12:
            tempInt = randint(0, 6)
            texture.noise_basis = noises[tempInt]
            texture.noise_scale = gauss(0.2, 1 / 30)
        else:
            tempInt = randint(0, 6)
            texture.noise_basis = noises[tempInt]
    elif texture.type == 'VORONOI':
        metrics = ['DISTANCE', 'DISTANCE_SQUARED', 'MANHATTAN', 'CHEBYCHEV',
                   'MINKOVSKY_HALF', 'MINKOVSKY_FOUR', 'MINKOVSKY']
        # Settings for first displacement level:
        if level == 0:
            tempInt = randint(0, 1)
            texture.distance_metric = metrics[tempInt]
            texture.noise_scale = gauss(0.625, 1 / 24)
            texture.contrast = 0.5
            texture.intensity = 0.7
        elif level == 2:
            texture.noise_scale = 0.15
            tempInt = randint(0, 6)
            texture.distance_metric = metrics[tempInt]
        elif level >= 12:
            tempInt = randint(0, 1)
            texture.distance_metric = metrics[tempInt]
            texture.noise_scale = gauss(0.125, 1 / 48)
            texture.contrast = 0.5
            texture.intensity = 0.7
        else:
            tempInt = randint(0, 6)
            texture.distance_metric = metrics[tempInt]

    return



# SPDX-FileCopyrightText: 2017-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

__author__ = "Nutti <nutti.metro@gmail.com>"
__status__ = "production"
__version__ = "6.7"
__date__ = "22 Sep 2022"


bl_info = {
    "name": "Magic UV",
    "author": "Nutti, Mifth, Jace Priester, kgeogeo, mem, imdjs, "
              "Keith (Wahooney) Boshoff, McBuff, MaxRobinot, "
              "Alexander Milovsky, Dusan Stevanovic, MatthiasThDs, "
              "theCryingMan, PratikBorhade302",
    "version": (6, 7, 0),
    "blender": (3, 4, 0),
    "location": "See Add-ons Preferences",
    "description": "UV Toolset. See Add-ons Preferences for details",
    "warning": "",
    "support": "COMMUNITY",
    "doc_url": "{BLENDER_MANUAL_URL}/addons/uv/magic_uv.html",
    "tracker_url": "https://github.com/nutti/Magic-UV",
    "category": "UV",
}


if "bpy" in locals():
    import importlib
    importlib.reload(common)
    importlib.reload(utils)
    utils.bl_class_registry.BlClassRegistry.cleanup()
    importlib.reload(op)
    importlib.reload(ui)
    importlib.reload(properties)
    importlib.reload(preferences)
else:
    import bpy
    from . import common
    from . import utils
    from . import op
    from . import ui
    from . import properties
    from . import preferences

import bpy


def register():
    utils.bl_class_registry.BlClassRegistry.register()
    properties.init_props(bpy.types.Scene)
    user_prefs = utils.compatibility.get_user_preferences(bpy.context)
    if user_prefs.addons['magic_uv'].preferences.enable_builtin_menu:
        preferences.add_builtin_menu()


def unregister():
    preferences.remove_builtin_menu()
    properties.clear_props(bpy.types.Scene)
    utils.bl_class_registry.BlClassRegistry.unregister()


if __name__ == "__main__":
    register()



# SPDX-FileCopyrightText: 2017-2022 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

import bpy
from bpy.types import Menu
from . import utils_core


class MasterSymmetryMenu(Menu):
    bl_label = "Symmetry Options"
    bl_idname = "VIEW3D_MT_sv3_master_symmetry_menu"

    @classmethod
    def poll(self, context):
        return utils_core.get_mode() in (
                        'SCULPT',
                        'VERTEX_PAINT',
                        'WEIGHT_PAINT',
                        'TEXTURE_PAINT',
                        'PARTICLE_EDIT',
                        )

    def draw(self, context):
        layout = self.layout

        if utils_core.get_mode() == 'PARTICLE_EDIT':
            layout.row().prop(context.active_object.data, "use_mirror_x",
                              text="Mirror X", toggle=True)

        elif utils_core.get_mode() == 'TEXTURE_PAINT':
            layout.row().prop(context.active_object, "use_mesh_mirror_x",
                              text="Symmetry X", toggle=True)
            layout.row().prop(context.active_object, "use_mesh_mirror_y",
                              text="Symmetry Y", toggle=True)
            layout.row().prop(context.active_object, "use_mesh_mirror_z",
                              text="Symmetry Z", toggle=True)
        else:
            layout.row().menu(SymmetryMenu.bl_idname)
            layout.row().menu(SymmetryRadialMenu.bl_idname)

            if utils_core.get_mode() == 'SCULPT':
                layout.row().prop(context.tool_settings.sculpt, "use_symmetry_feather",
                                  toggle=True)


class SymmetryMenu(Menu):
    bl_label = "Symmetry"
    bl_idname = "VIEW3D_MT_sv3_symmetry_menu"

    def draw(self, context):
        layout = self.layout

        layout.row().label(text="Symmetry")
        layout.row().separator()

        layout.row().prop(context.active_object, "use_mesh_mirror_x",
                          text="Symmetry X", toggle=True)
        layout.row().prop(context.active_object, "use_mesh_mirror_y",
                          text="Symmetry Y", toggle=True)
        layout.row().prop(context.active_object, "use_mesh_mirror_z",
                          text="Symmetry Z", toggle=True)


class SymmetryRadialMenu(Menu):
    bl_label = "Radial"
    bl_idname = "VIEW3D_MT_sv3_symmetry_radial_menu"

    def draw(self, context):
        layout = self.layout

        layout.row().label(text="Radial")
        layout.row().separator()

        mode_tool_settings = getattr(context.tool_settings, utils_core.get_mode().lower())

        layout.column().prop(mode_tool_settings, "radial_symmetry", text="", slider=True)


classes = (
    MasterSymmetryMenu,
    SymmetryMenu,
    SymmetryRadialMenu
    )

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
